//************************************************
//* Matrix keyboard decoder
//* Test application
//*
//* Filename:	myCellPhone.c
//* Version:	1.0
//*
//* Device:		ATmega162
//* Clock:		Internal 1.000 MHz RC oscillator
//*
//* This test application implements a cellphone-like
//* keyboard. Using three one-shot alternation keys,
//* the whole alphabet is available. A capslock
//* function is also implemented.
//* Miscellaneous status information is output on Port A
//* for LED connection on e.g. STK500
//*
//************************************************
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <ctype.h>
#include "keymatrix.h"
#include "Keyboard.h"
#pragma language=extended



/*** Macros and definitions ***/
#define CPUFREQ 8000.000						// CPU frequency 1.000 MHz
#define BAUDRATE 19.200							// USART Baud rate 19.2 kbps

#define BAUDFACTOR CPUFREQ/(16*BAUDRATE)-1		// Factor used in UBRRL



//*** Character conversion table ***
//*
//* keyboard layout:
//*
//* 1 2 3 A . . . . 
//* 4 5 6 B . . . .
//* 7 8 9 C . . . .
//* * 0 # D . . . .
//* . . . . . . . . 
//* . . . . . . . . 
//* . . . . . . . . 
//* . . . . . . . . 
//*
//* Four rightmost columns and lower four rows
//* not connected. The application uses a 4x4 keypad only.
//*
//* Alternation key functions:
//*
//* none - ordinary digit key function
//* A - use first character on digit key
//* B - use second character on digit key
//* C - use third character on digit key
//* D - caps lock, use uppercase letters
//*
//* The digit keys thus represent characters
//* when alternation keys are used:
//*
//* 1 - ABC, 2 - DEF, 3 - GHI
//* 4 - JKL, 5 - MNO, 6 - PQR
//* 7 - STU, 8 - VWX, 9 - YZ-
//* * - ***, 0 - .?!, # - ## #

TASK_LIST
{
	{ Task: USB_USBTask          , TaskStatus: TASK_STOP },
	{ Task: USB_Keyboard_Report  , TaskStatus: TASK_STOP },
};


char characters[2][40] PROGMEM = { 
		           { '+', '5', 0, 0, 0, 0, 0, 0,	// No alternation
                             '-', '4', 0, 0, 0, 0, 0, 0,
                             'o', '3', 0, 0, 0, 0, 0, 0,
                             'n', '2', 0, 0, 0, 0, 0, 0, 
			     'f', '1', 0, 0, 0, 0, 0, 0 },
                             
                           { 'a', 'u', 0, 0, 0, 0, 0, 0,	// First character
                             'j', 'v', 0, 0, 0, 0, 0, 0,
                             's', 'w', 0, 0, 0, 0, 0, 0,
                             '*', 'x', 0, 0, 0, 0, 0, 0,
			     '*', 'y', 0, 0, 0, 0, 0, 0 } 
			   };
                             
                           //{ 'b', 'e', 'h', 0, 0, 0, 0, 0,	// Second character
                           //'k', 'n', 'q', 0, 0, 0, 0, 0,
                           //'t', 'w', 'z', 0, 0, 0, 0, 0,
                           //'*', '?', '#', 0, 0, 0, 0, 0 },
                             
                           //{ 'c', 'f', 'i', 0, 0, 0, 0, 0,	// Third character
                           //  'l', 'o', 'r', 0, 0, 0, 0, 0,
                           //  'u', 'x', '-', 0, 0, 0, 0, 0,
                           //  '*', '!', '#', 0, 0, 0, 0, 0 }
                         //};
                             


/*** Transmit one byte using USART1 ***/
/*void sendChar( char data )
{
	if( data )									// Don't transmit 0
	{
		while( !(UCSR1A & (1<<UDRE1)) );		// Wait for data register ready flag

		UCSR1A |= (1<<TXC1);					// Clear transmit complete flag

		UDR1 = data;							// Transmit byte
		PORTA = ~data;							// Put to port A also, connected to STK500 LEDs

		while( !(UCSR1A & (1<<TXC1)) );			// Wait for transmit complete
	}											// before reentering sleep mode.
}*/												// This is only reqired when using sleep modes.



/*** Convert scancode to character ***/
char convertKey(void)
{
	char tempChar;

	if( key_code.altKey0 )				// First char ?
	{
		tempChar = characters[1][key_code.scan];
		
	} else if( key_code.altKey1 )			// Second char ?
	{
		tempChar = characters[2][key_code.scan];
		
	} else if( key_code.altKey2 )			// Third char ?
	{
		tempChar = characters[3][key_code.scan];
			
	} else						// No alternation ?
	{
		tempChar = characters[0][key_code.scan];
	}


	if( key_code.lckKey0 )				// Uppercase if caps lock
	{
		tempChar = toupper( tempChar );
	}
	
	
	return tempChar;
}



/*** Main test application loop ***/
int main(void)
{
	/* Disable watchdog if enabled by bootloader/fuses */
	MCUSR &= ~(1 << WDRF);
	wdt_disable();

	/* Disable Clock Division */
	SetSystemClockPrescaler(0);

	/* Enable sleep mode */
	MCUCR |= (1<<SE);
	

	/* Init USART */
	//UCSR1B |= (1<<TXEN1);						// Enable transmitter
	//UBRR1L = BAUDFACTOR;						// Set baud rate
	
	/* Init LEDs */
	LEDs_Init();

	/* Init keyboard driver */
	key_stop();
	key_init();
	sei();

	/* Indicate USB not ready */
	LEDs_SetAllLEDs(LEDS_LED1 | LEDS_LED3);

	/* Initialize Scheduler so that it can be used */
	Scheduler_Init();

	/* Initialize USB Subsystem */
	USB_Init();

	/* Scheduling - routine never returns, so put this last in the main function */
	Scheduler_Start();
	
	/* Init ports */
	//DDRA = 0xFF;								// Port A all output
	//PORTA = 0;
	
DDRA = 0xFF;	//willkürlich, kein Anschluss an Port A?!

}

EVENT_HANDLER(USB_Connect)
{
	/* Start USB management task */
	Scheduler_SetTaskMode(USB_USBTask, TASK_RUN);

	/* Indicate USB enumerating */
	LEDs_SetAllLEDs(LEDS_LED1 | LEDS_LED4);
}

EVENT_HANDLER(USB_Disconnect)
{
	/* Stop running keyboard reporting and USB management tasks */
	Scheduler_SetTaskMode(USB_Keyboard_Report, TASK_STOP);
	Scheduler_SetTaskMode(USB_USBTask, TASK_STOP);

	/* Indicate USB not ready */
	LEDs_SetAllLEDs(LEDS_LED1 | LEDS_LED3);
}

EVENT_HANDLER(USB_CreateEndpoints)
{
	/* Setup Keyboard Report Endpoints */
	Endpoint_ConfigureEndpoint(KEYBOARD_EPNUM, EP_TYPE_INTERRUPT,
		                       ENDPOINT_DIR_IN, KEYBOARD_EPSIZE,
	                           ENDPOINT_BANK_SINGLE);

	/* Setup Keyboard Report Endpoints */
	Endpoint_ConfigureEndpoint(KEYBOARD_LEDS_EPNUM, EP_TYPE_INTERRUPT,
		                       ENDPOINT_DIR_OUT, KEYBOARD_EPSIZE,
	                           ENDPOINT_BANK_SINGLE);

	/* Indicate USB connected and ready */
	LEDs_SetAllLEDs(LEDS_LED2 | LEDS_LED4);

	/* Start Keyboard reporting task */
	Scheduler_SetTaskMode(USB_Keyboard_Report, TASK_RUN);
}

TASK(USB_Keyboard_Report)
{
	char tempChar;
	
	key_get();						// Wait for keypress

	tempChar = convertKey();				// Convert to character
	
/*	while( key_code.scan != 0xFF )			// Uncomment to implement simple key repetition
	{
			sendChar( tempChar );				// Transmit character until key released
	}*/


	USB_KeyboardReport_Data_t KeyboardReportData = {Modifier: 0, KeyCode: 0};
	uint8_t                   KeyStatus_LCL      = tempChar;

	if (KeyStatus_LCL == '+')
	  KeyboardReportData.KeyCode = 0x57; // +

	if (KeyStatus_LCL == '5')
	  KeyboardReportData.KeyCode = 0x22; // 5

	if (KeyStatus_LCL == '-')
	  KeyboardReportData.KeyCode = 0x56; // -

	if (KeyStatus_LCL == '4')
	  KeyboardReportData.KeyCode = 0x21; // 4

	if (KeyStatus_LCL == 'o')
	  KeyboardReportData.KeyCode = 0x12; // o

	if (KeyStatus_LCL == '3')
	  KeyboardReportData.KeyCode = 0x20; // 3

	if (KeyStatus_LCL == 'n')
	  KeyboardReportData.KeyCode = 0x11; // n

	if (KeyStatus_LCL == '2')
	  KeyboardReportData.KeyCode = 0x1F; // 2

	if (KeyStatus_LCL == 'f')
	  KeyboardReportData.KeyCode = 0x09; // f

	if (KeyStatus_LCL == '1')
	  KeyboardReportData.KeyCode = 0x1E; // 1

	/* Check if the USB System is connected to a Host */
	if (USB_IsConnected)
	{
		/* Select the Keyboard Report Endpoint */
		Endpoint_SelectEndpoint(KEYBOARD_EPNUM);

		/* Check if Keyboard Endpoint Ready for Read/Write */
		if (Endpoint_ReadWriteAllowed())
		{
			/* Write Keyboard Report Data */
			Endpoint_Write_Byte(KeyboardReportData.Modifier);
			Endpoint_Write_Byte(0x00);
			Endpoint_Write_Byte(KeyboardReportData.KeyCode);			
			Endpoint_Write_Byte(0x00);
			Endpoint_Write_Byte(0x00);
			Endpoint_Write_Byte(0x00);
			Endpoint_Write_Byte(0x00);
			Endpoint_Write_Byte(0x00);

			/* Handshake the IN Endpoint - send the data to the host */
			Endpoint_FIFOCON_Clear();
		}

		/* Select the Keyboard LED Report Endpoint */
		Endpoint_SelectEndpoint(KEYBOARD_LEDS_EPNUM);

		/* Check if Keyboard LED Endpoint Ready for Read/Write */
		if (Endpoint_ReadWriteAllowed())
		{
			/* Read in the LED report from the host */
			uint8_t LEDStatus = Endpoint_Read_Byte();
			uint8_t LEDMask   = LEDS_LED2;
			
			if (LEDStatus & 0x01) // NUM Lock
			  LEDMask |= LEDS_LED1;
			
			if (LEDStatus & 0x02) // CAPS Lock
			  LEDMask |= LEDS_LED3;

			if (LEDStatus & 0x04) // SCROLL Lock
			  LEDMask |= LEDS_LED4;

			/* Set the status LEDs to the current Keyboard LED status */
			LEDs_SetAllLEDs(LEDMask);

			/* Handshake the OUT Endpoint - clear endpoint and ready for next report */
			Endpoint_FIFOCON_Clear();
		}
	}
}